from datetime import datetime
from typing import Dict, Generator, List

from .assert_funcs import AssertCollectionStatusNormal, AssertCollectionUrl
from .basic_apis import (GetCollectionArticlesJsonDataApi,
                         GetCollectionEditorsJsonDataApi,
                         GetCollectionJsonDataApi,
                         GetCollectionRecommendedWritersJsonDataApi,
                         GetCollectionSubscribersJsonDataApi)
from .convert import CollectionUrlToCollectionSlug


def GetCollectionName(collection_url: str, disable_check: bool = False) -> str:
    """获取专题名称

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: 专题名称
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["title"]
    return result


def GetCollectionAvatarUrl(collection_url: str, disable_check: bool = False) -> str:
    """获取专题头像链接

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: 专题头像链接
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["image"]
    return result


def GetCollectionIntroductionText(collection_url: str, disable_check: bool = False) -> str:
    """获取纯文本格式的专题简介

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: 纯文本格式的专题简介
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["content_without_html"]
    return result


def GetCollectionIntroductionHtml(collection_url: str, disable_check: bool = False) -> str:
    """获取 Html 格式的专题简介

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: Html 格式的专题简介
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["content_in_full"]
    return result


def GetCollectionArticlesCount(collection_url: str, disable_check: bool = False) -> int:
    """获取专题中的文章数量

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 专题中的文章数量
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["notes_count"]
    return result


def GetCollectionSubscribersCount(collection_url: str, disable_check: bool = False) -> int:
    """获取专题的订阅者数量

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 专题的订阅者数量
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["subscribers_count"]
    return result


def GetCollectionArticlesUpdateTime(collection_url: str, disable_check: bool = False) -> datetime:
    """获取专题文章更新时间

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        datetime: 专题文章更新时间
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = datetime.fromtimestamp(json_obj["newly_added_at"])
    return result


def GetCollectionInformationUpdateTime(collection_url: str, disable_check: bool = False) -> datetime:
    """获取专题信息更新时间

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        datetime: 专题信息更新时间
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result


def GetCollectionOwnerInfo(collection_url: str, disable_check: bool = False) -> Dict:
    """获取专题的所有者信息

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        Dict: 用户信息
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = {
        "uid": json_obj["owner"]["id"],
        "name": json_obj["owner"]["nickname"],
        "uslug": json_obj["owner"]["slug"]
    }
    return result


def GetCollectionEditorsInfo(collection_id: int, page: int = 1) -> List[Dict]:
    """获取专题编辑信息

    Args:
        collection_id (int): 专题 ID
        page (int, optional): 页码. Defaults to 1.

    Returns:
        List[Dict]: 专题编辑信息
    """
    json_obj = GetCollectionEditorsJsonDataApi(collection_id, page=page)
    result = []
    for item in json_obj["editors"]:
        item_data = {
            "uslug": item["slug"],
            "name": item["nickname"],
            "avatar_url": item["avatar_source"]
        }
        result.append(item_data)
    return result


def GetCollectionRecommendedWritersInfo(collection_id: int, page: int = 1, count: int = 20) -> List[Dict]:
    """获取专题推荐作者信息

    Args:
        collection_id (int): 专题 ID
        page (int, optional): 页码. Defaults to 1.
        count (int, optional): 每次返回的结果数量. Defaults to 20.

    Returns:
        List[Dict]: 专题推荐作者信息
    """
    json_obj = GetCollectionRecommendedWritersJsonDataApi(collection_id, page=page, count=count)
    result = []
    for item in json_obj["users"]:
        item_data = {
            "uid": item["id"],
            "uslug": item["slug"],
            "name": item["nickname"],
            "avatar_url": item["avatar_source"],
            "collection_name": item["collection_name"],
            "likes_count": item["total_likes_count"],
            "words_count": item["total_wordage"]
        }
        result.append(item_data)
    return result


def GetCollectionSubscribersInfo(collection_id: int, start_sort_id: int = None) -> List[Dict]:
    """获取专题关注者信息

    Args:
        collection_id (int): 专题 ID
        start_sort_id (int): 起始序号，等于上一条数据的序号

    Returns:
        List[Dict]: 关注者信息
    """
    json_obj = GetCollectionSubscribersJsonDataApi(collection_id, max_sort_id=start_sort_id)
    result = []
    for item in json_obj:
        item_data = {
            "uslug": item["slug"],
            "name": item["nickname"],
            "avatar_url": item["avatar_source"],
            "sort_id": item["like_id"],
            "subscribe_time": datetime.fromisoformat(item["subscribed_at"])
        }
        result.append(item_data)
    return result


def GetCollectionArticlesInfo(collection_url: str, page: int = 1,
                              count: int = 10, sorting_method: str = "time",
                              disable_check: bool = False) -> List[Dict]:
    """获取专题文章信息

    Args:
        collection_url (str): 专题 Url
        page (int, optional): 页码. Defaults to 1.
        count (int, optional): 每次返回的数据数量. Defaults to 10.
        sorting_method (str, optional): 排序方法，"time" 为按照发布时间排序，
        "comment_time" 为按照最近评论时间排序，"hot" 为按照热度排序. Defaults to "time".
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 文章信息
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    order_by = {
        "time": "added_at",
        "comment_time": "commented_at",
        "hot": "top"
    }[sorting_method]
    json_obj = GetCollectionArticlesJsonDataApi(CollectionUrlToCollectionSlug(collection_url),
                                                page=page, count=count, order_by=order_by)
    result = []
    for item in json_obj:
        item_data = {
            "aid": item["object"]["data"]["id"],
            "title": item["object"]["data"]["title"],
            "aslug": item["object"]["data"]["slug"],
            "release_time": datetime.fromisoformat(item["object"]["data"]["first_shared_at"]),
            "first_image_url": item["object"]["data"]["list_image_url"],
            "summary": item["object"]["data"]["public_abbr"],
            "views_count": item["object"]["data"]["views_count"],
            "likes_count": item["object"]["data"]["likes_count"],
            "paid": item["object"]["data"]["paid"],
            "commentable": item["object"]["data"]["commentable"],
            "user": {
                "uid": item["object"]["data"]["user"]["id"],
                "name": item["object"]["data"]["user"]["nickname"],
                "uslug": item["object"]["data"]["user"]["slug"],
                "avatar_url": item["object"]["data"]["user"]["avatar"]
            },
            "total_fp_amount": item["object"]["data"]["total_fp_amount"] / 1000,
            "comments_count": item["object"]["data"]["public_comments_count"],
            "rewards_count": item["object"]["data"]["total_rewards_count"]
        }
        result.append(item_data)
    return result


def GetCollectionAllBasicData(collection_url: str, disable_check: bool = False) -> Dict:
    """获取专题的所有基础信息

    Args:
        collection_url (str): 专题 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        Dict: 专题基础信息
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    result = {}
    json_obj = GetCollectionJsonDataApi(collection_url)

    result["name"] = json_obj["title"]
    result["avatar_url"] = json_obj["image"]
    result["introduction_text"] = json_obj["content_without_html"]
    result["introduction_html"] = json_obj["content_in_full"]
    result["articles_count"] = json_obj["notes_count"]
    result["subscribers_count"] = json_obj["subscribers_count"]
    result["articles_update_time"] = datetime.fromtimestamp(json_obj["newly_added_at"])
    result["information_update_time"] = datetime.fromtimestamp(json_obj["last_updated_at"])
    result["owner_info"] = {
        "uid": json_obj["owner"]["id"],
        "name": json_obj["owner"]["nickname"],
        "uslug": json_obj["owner"]["slug"]
    }
    return result


def GetCollectionAllEditorsInfo(collection_id: int, max_count: int = None) -> Generator[Dict, None, None]:
    """获取专题的所有编辑信息

    Args:
        collection_id (int): 专题 ID
        max_count (int, optional): 获取的专题编辑信息数量上限，Defaults to None.

    Yields:
        Iterator[Dict], None, None]: 编辑信息
    """
    page = 1
    now_count = 0
    while True:
        result = GetCollectionEditorsInfo(collection_id, page)
        if result:
            page += 1
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if now_count == max_count:
                    return


def GetCollectionAllRecommendedWritersInfo(collection_id: int, count: int = 20, max_count: int = None) -> Generator[Dict, None, None]:
    """获取专题的所有推荐作者信息

    Args:
        collection_id (int): 专题 ID
        count (int, optional): 单次获取的数据数量，会影响性能. Defaults to 20.
        max_count (int, optional): 获取的专题推荐作者信息数量上限，Defaults to None.

    Yields:
        Iterator[Dict], None, None]: 推荐作者信息
    """
    page = 1
    now_count = 0
    while True:
        result = GetCollectionRecommendedWritersInfo(collection_id, page, count)
        if result:
            page += 1
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if now_count == max_count:
                    return


def GetCollectionAllSubscribersInfo(collection_id: int, max_count: int = None) -> Generator[Dict, None, None]:
    """获取专题的所有关注者信息

    Args:
        collection_id (int): 专题 ID
        max_count (int, optional): 获取的专题关注者信息数量上限，Defaults to None.

    Yields:
        Iterator[Dict], None, None]: 关注者信息
    """
    start_sort_id = None
    now_count = 0
    while True:
        result = GetCollectionSubscribersInfo(collection_id, start_sort_id)
        if result:
            start_sort_id = result[-1]["sort_id"]
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if now_count == max_count:
                    return


def GetCollectionAllArticlesInfo(collection_url: str, count: int = 10,
                                 sorting_method: str = "time", max_count: int = None,
                                 disable_check: bool = False) -> Generator[Dict, None, None]:
    """获取专题的所有文章信息

    Args:
        collection_url (str): 专题 Url
        count (int, optional): 单次获取的数据数量，会影响性能. Defaults to 10.
        sorting_method (str, optional): 排序方法，"time" 为按照发布时间排序，
        "comment_time" 为按照最近评论时间排序，"hot" 为按照热度排序. Defaults to "time".
        max_count (int, optional): 获取的专题文章信息数量上限，Defaults to None.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Yields:
        Iterator[Dict], None, None]: 文章信息
    """
    if not disable_check:
        AssertCollectionUrl(collection_url)
        AssertCollectionStatusNormal(collection_url)
    page = 1
    now_count = 0
    while True:
        result = GetCollectionArticlesInfo(collection_url, page, count, sorting_method, disable_check=True)
        if result:
            page += 1
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if now_count == max_count:
                    return



from typing import Dict, List, Union

from httpx import get as httpx_get
from httpx import post as httpx_post
from lxml import etree

from .headers import (BeikeIsland_request_header, PC_header,
                      jianshu_request_header, mobile_header)

try:
    from ujson import loads as json_loads
except ImportError:
    from json import loads as json_loads


def GetArticleJsonDataApi(article_url: str) -> Dict:
    request_url = article_url.replace("https://www.jianshu.com/", "https://www.jianshu.com/asimov/")
    source = httpx_get(request_url, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetArticleHtmlJsonDataApi(article_url: str) -> Dict:
    source = httpx_get(article_url, headers=PC_header).content
    html_obj = etree.HTML(source)
    json_obj = json_loads(html_obj.xpath("//script[@id='__NEXT_DATA__']/text()")[0])
    return json_obj


def GetArticleCommentsJsonDataApi(article_id: int, page: int, count: int,
                                  author_only: bool, order_by: str) -> Dict:
    params = {
        "page": page,
        "count": count,
        "author_only": author_only,
        "order_by": order_by
    }
    request_url = f"https://www.jianshu.com/shakespeare/notes/{article_id}/comments"
    source = httpx_get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetBeikeIslandTradeRankListJsonDataApi(ranktype: Union[int, None], pageIndex: Union[int, None]) -> Dict:
    params = {
        "ranktype": ranktype,
        "pageIndex": pageIndex
    }
    source = httpx_post("https://www.beikeisland.com/api/Trade/getTradeRankList",
                        headers=BeikeIsland_request_header, json=params).content
    json_obj = json_loads(source)
    return json_obj


def GetBeikeIslandTradeListJsonDataApi(pageIndex: int, retype: int):
    params = {
        "pageIndex": pageIndex,
        "retype": retype
    }
    source = httpx_post("https://www.beikeisland.com/api/Trade/getTradeList",
                        headers=BeikeIsland_request_header, json=params).content
    json_obj = json_loads(source)
    return json_obj


def GetCollectionJsonDataApi(collection_url: str) -> Dict:
    request_url = collection_url.replace("https://www.jianshu.com/c/", "https://www.jianshu.com/asimov/collections/slug/")
    source = httpx_get(request_url, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetCollectionEditorsJsonDataApi(collection_id: int, page: int) -> Dict:
    request_url = f"https://www.jianshu.com/collections/{collection_id}/editors"
    params = {
        "page": page
    }
    source = httpx_get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetCollectionRecommendedWritersJsonDataApi(collection_id: int, page: int, count: int) -> Dict:
    params = {
        "collection_id": collection_id,
        "page": page,
        "count": count
    }
    source = httpx_get("https://www.jianshu.com/collections/recommended_users",
                       params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetCollectionSubscribersJsonDataApi(collection_id: int, max_sort_id: int) -> Dict:
    request_url = f"https://www.jianshu.com/collection/{collection_id}/subscribers"
    params = {
        "max_sort_id": max_sort_id
    }
    source = httpx_get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetCollectionArticlesJsonDataApi(collection_slug: str, page: int, count: int, order_by: str) -> Dict:
    request_url = f"https://www.jianshu.com/asimov/collections/slug/{collection_slug}/public_notes"
    params = {
        "page": page,
        "count": count,
        "order_by": order_by
    }
    source = httpx_get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetIslandJsonDataApi(island_url: str) -> Dict:
    request_url = island_url.replace("https://www.jianshu.com/g/", "https://www.jianshu.com/asimov/groups/")
    source = httpx_get(request_url, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetIslandPostsJsonDataApi(group_slug: str, max_id: int,
                              count: int, topic_id: int, order_by: str):
    params = {
        "group_slug": group_slug,
        "order_by": order_by,
        "max_id": max_id,
        "count": count,
        "topic_id": topic_id
    }
    source = httpx_get("https://www.jianshu.com/asimov/posts",
                       params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetNotebookJsonDataApi(notebook_url: str) -> Dict:
    request_url = notebook_url.replace("https://www.jianshu.com/", "https://www.jianshu.com/asimov/")
    source = httpx_get(request_url, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetNotebookArticlesJsonDataApi(notebook_url: str, page: int,
                                   count: int, order_by: str) -> Dict:
    request_url = notebook_url.replace("https://www.jianshu.com/nb/",
                                       "https://www.jianshu.com/asimov/notebooks/") + "/public_notes/"
    params = {
        "page": page,
        "count": count,
        "order_by": order_by
    }
    source = httpx_get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetAssetsRankJsonDataApi(max_id: int, since_id: int) -> Dict:
    params = {
        "max_id": max_id,
        "since_id": since_id
    }
    source = httpx_get("https://www.jianshu.com/asimov/fp_rankings", params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetDailyArticleRankListJsonDataApi() -> Dict:
    source = httpx_get("https://www.jianshu.com/asimov/daily_activity_participants/rank", headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetArticlesFPRankListJsonDataApi(date: str, type_: str) -> Dict:  # 避免覆盖内置函数
    params = {
        "date": date,
        "type": type_
    }
    source = httpx_get("https://www.jianshu.com/asimov/fp_rankings/voter_notes", params=params, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetUserJsonDataApi(user_url: str) -> Dict:
    request_url = user_url.replace("https://www.jianshu.com/u/", "https://www.jianshu.com/asimov/users/slug/")
    source = httpx_get(request_url, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetUserPCHtmlDataApi(user_url: str) -> Dict:
    source = httpx_get(user_url, headers=PC_header).content
    html_obj = etree.HTML(source)
    return html_obj


def GetUserCollectionsAndNotebooksJsonDataApi(user_url: str, user_slug: str) -> Dict:
    request_url = user_url.replace("/u/", "/users/") + "/collections_and_notebooks"
    params = {
        "slug": user_slug
    }
    source = httpx_get(request_url, headers=jianshu_request_header, params=params).content
    json_obj = json_loads(source)
    return json_obj


def GetUserArticlesListJsonDataApi(user_url: str, page: int,
                                   count: int, order_by: str) -> Dict:
    request_url = user_url.replace("/u/", "/asimov/users/slug/") + "/public_notes"
    params = {
        "page": page,
        "count": count,
        "order_by": order_by
    }
    source = httpx_get(request_url, headers=jianshu_request_header, params=params).content
    json_obj = json_loads(source)
    return json_obj


def GetUserFollowingListHtmlDataApi(user_url: str, page: int):
    request_url = user_url.replace("/u/", "/users/") + "/following"
    params = {
        "page": page
    }
    source = httpx_get(request_url, headers=PC_header, params=params).content
    html_obj = etree.HTML(source)
    return html_obj


def GetUserFollowersListHtmlDataApi(user_url: str, page: int):
    request_url = user_url.replace("/u/", "/users/") + "/followers"
    params = {
        "page": page
    }
    source = httpx_get(request_url, headers=PC_header, params=params).content
    html_obj = etree.HTML(source)
    return html_obj


def GetUserNextAnniversaryDayHtmlDataApi(user_slug: str):
    request_url = f"https://www.jianshu.com/mobile/u/{user_slug}/anniversary"
    source = httpx_get(request_url, headers=mobile_header).content
    html_obj = etree.HTML(source)
    return html_obj


def GetIslandPostJsonDataApi(post_slug: str) -> List[Dict]:
    request_url = f"https://www.jianshu.com/asimov/posts/{post_slug}"
    source = httpx_get(request_url, headers=jianshu_request_header).content
    json_obj = json_loads(source)
    return json_obj


def GetUserTimelineHtmlDataApi(uslug: str, max_id: int) -> Dict:
    request_url = f"https://www.jianshu.com/users/{uslug}/timeline"
    params = {
        "max_id": max_id
    }
    source = httpx_get(request_url, headers=PC_header, params=params).content
    html_obj = etree.HTML(source)
    return html_obj



from datetime import datetime
from typing import Dict, Generator, List

from .assert_funcs import AssertNotebookStatusNormal, AssertNotebookUrl
from .basic_apis import GetNotebookArticlesJsonDataApi, GetNotebookJsonDataApi


def GetNotebookName(notebook_url: str, disable_check: bool = False) -> str:
    """获取文集名称

    Args:
        notebook_url (str): 文集 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: 文集名称
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["name"]
    return result


def GetNotebookArticlesCount(notebook_url: str, disable_check: bool = False) -> int:
    """获取文集中的文章数量

    Args:
        notebook_url (str): 文集 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 文章数量
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["notes_count"]
    return result


def GetNotebookAuthorInfo(notebook_url: str, disable_check: bool = False) -> Dict:
    """获取文集作者的信息

    Args:
        notebook_url (str): 文集 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        Dict: 作者信息
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = {
        "name": json_obj["user"]["nickname"],
        "uslug": json_obj["user"]["slug"],
        "avatar_url": json_obj["user"]["avatar"]
    }
    return result


def GetNotebookWordage(notebook_url: str, disable_check: bool = False) -> int:
    """获取文集中所有文章的总字数

    Args:
        notebook_url (str): 文集 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 文集中的文章总字数
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["wordage"]
    return result


def GetNotebookSubscribersCount(notebook_url: str, disable_check: bool = False) -> int:
    """获取文集的订阅者数量

    Args:
        notebook_url (str): 文集 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 文集订阅者数量
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["subscribers_count"]
    return result


def GetNotebookUpdateTime(notebook_url: str, disable_check: bool = False) -> datetime:
    """获取文集的更新时间

    Args:
        notebook_url (str): 文集 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        datetime: 更新时间
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result


def GetNotebookArticlesInfo(notebook_url: str, page: int = 1,
                            count: int = 10, sorting_method: str = "time",
                            disable_check: bool = False) -> List[Dict]:
    """获取文集中的文章信息

    Args:
        notebook_url (str): 文集 Url
        page (int, optional): 页码. Defaults to 1.
        count (int, optional): 每次返回的数据数量. Defaults to 10.
        sorting_method (str, optional): 排序方法，"time" 为按照发布时间排序，
        "comment_time" 为按照最近评论时间排序，"hot" 为按照热度排序. Defaults to "time".
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 文章信息
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    order_by = {
        "time": "added_at",
        "comment_time": "commented_at",
        "hot": "top"
    }[sorting_method]
    json_obj = GetNotebookArticlesJsonDataApi(notebook_url=notebook_url,
                                              page=page, count=count, order_by=order_by)
    result = []
    for item in json_obj:
        item_data = {
            "aid": item["object"]["data"]["id"],
            "title": item["object"]["data"]["title"],
            "aslug": item["object"]["data"]["slug"],
            "release_time": datetime.fromisoformat(item["object"]["data"]["first_shared_at"]),
            "first_image_url": item["object"]["data"]["list_image_url"],
            "summary": item["object"]["data"]["public_abbr"],
            "views_count": item["object"]["data"]["views_count"],
            "likes_count": item["object"]["data"]["likes_count"],
            "is_top": item["object"]["data"]["is_top"],
            "paid": item["object"]["data"]["paid"],
            "commentable": item["object"]["data"]["commentable"],
            "user": {
                "uid": item["object"]["data"]["user"]["id"],
                "name": item["object"]["data"]["user"]["nickname"],
                "uslug": item["object"]["data"]["user"]["slug"],
                "avatar_url": item["object"]["data"]["user"]["avatar"]
            },
            "total_fp_amount": item["object"]["data"]["total_fp_amount"] / 1000,
            "comments_count": item["object"]["data"]["public_comments_count"],
            "rewards_count": item["object"]["data"]["total_rewards_count"]
        }
        result.append(item_data)
    return result


def GetNotebookAllBasicData(notebook_url: str, disable_check: bool = False) -> Dict:
    """获取文集的所有基础信息

    Args:
        notebook_url (str): 文集 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        Dict: 文集基础信息
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    result = {}
    json_obj = GetNotebookJsonDataApi(notebook_url)

    result["name"] = json_obj["name"]
    result["author_info"] = {
        "name": json_obj["user"]["nickname"],
        "uslug": json_obj["user"]["slug"],
        "avatar_url": json_obj["user"]["avatar"]
    }
    result["articles_count"] = json_obj["notes_count"]
    result["wordage"] = json_obj["wordage"]
    result["subscribers_count"] = json_obj["subscribers_count"]
    result["update_time"] = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result


def GetNotebookAllArticlesInfo(notebook_url: str, count: int = 10, sorting_method: str = "time",
                               max_count: int = None, disable_check: bool = False) -> Generator[Dict, None, None]:
    """获取文集中的全部文章信息

    Args:
        notebook_url (str): 文集 Url
        count (int, optional): 单次获取的数据数量，会影响性能. Defaults to 10.
        sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".
        max_count (int, optional): 获取的文集文章信息数量上限，Defaults to None.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Yields:
        Iterator[Dict], None, None]: 文章信息
    """
    if not disable_check:
        AssertNotebookUrl(notebook_url)
        AssertNotebookStatusNormal(notebook_url)
    page = 1
    now_count = 0
    while True:
        result = GetNotebookArticlesInfo(notebook_url, page, count, sorting_method, disable_check=True)
        if result:
            page += 1
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if max_count == now_count:
                    return



class InputError(Exception):
    """传入参数错误时抛出此异常
    """
    pass


class APIError(Exception):
    """API 受限导致无法获取信息时抛出此异常
    """
    pass


class ResourceError(Exception):
    """访问的资源状态异常时抛出此异常
    """



from datetime import datetime
from re import findall
from typing import Dict, Generator, List

from lxml import etree

from .assert_funcs import AssertUserStatusNormal, AssertUserUrl
from .basic_apis import (GetUserArticlesListJsonDataApi,
                         GetUserCollectionsAndNotebooksJsonDataApi,
                         GetUserFollowersListHtmlDataApi,
                         GetUserFollowingListHtmlDataApi, GetUserJsonDataApi,
                         GetUserNextAnniversaryDayHtmlDataApi,
                         GetUserPCHtmlDataApi, GetUserTimelineHtmlDataApi)
from .convert import (ArticleSlugToArticleUrl, CollectionSlugToCollectionUrl,
                      NotebookSlugToNotebookUrl, UserSlugToUserUrl,
                      UserUrlToUserSlug)
from .exceptions import APIError


def GetUserName(user_url: str, disable_check: bool = False) -> str:
    """获取用户昵称

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: 用户昵称
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["nickname"]
    return result


def GetUserGender(user_url: str, disable_check: bool = False) -> int:
    """获取用户性别

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 用户性别，0 为未知，1 为男，2 为女
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["gender"]
    if result == 3:  # 某些未设置性别的账号性别值为 3，怀疑为简书系统遗留问题
        result = 0  # 3 也代表性别未知
    return result


def GetUserFollowersCount(user_url: str, disable_check: bool = False) -> int:
    """获取用户关注人数

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 用户关注人数
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["following_users_count"]
    return result


def GetUserFansCount(user_url: str, disable_check: bool = False) -> int:
    """获取用户粉丝数

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 用户粉丝数
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["followers_count"]
    return result


def GetUserArticlesCount(user_url: str, disable_check: bool = False) -> int:
    """获取用户文章数

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 用户文章数
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    html_obj = GetUserPCHtmlDataApi(user_url)
    result = html_obj.xpath("//div[@class='info']/ul/li[3]/div[@class='meta-block']/a/p")[0].text
    result = int(result)
    return result


def GetUserWordage(user_url: str, disable_check: bool = False) -> int:
    """获取用户文章总字数

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 用户文章总字数
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["total_wordage"]
    return result


def GetUserLikesCount(user_url: str, disable_check: bool = False) -> int:
    """获取用户被喜欢数

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        int: 用户被喜欢数
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["total_likes_count"]
    return result


def GetUserAssetsCount(user_url: str, disable_check: bool = False) -> float:
    """获取用户总资产

    # ! 当用户资产大于 10000 时，结果的精确度将下降到 1000
    # ! 当用户没有文章时，该函数将抛出 APIError 异常

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Raises:
        APIError: 由于用户没有文章导致无法获取总资产信息时抛出此异常

    Returns:
        float: 用户总资产
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    html_obj = GetUserPCHtmlDataApi(user_url)
    try:
        result = html_obj.xpath("//div[@class='info']/ul/li[6]/div[@class='meta-block']/p")[0].text
    except IndexError:
        raise APIError("受 API 限制（用户无文章时无法获取总资产信息），无法获取该用户的总资产")
    result = float(result.replace(".", "").replace("w", "000"))
    return result


def GetUserFPCount(user_url: str, disable_check: bool = False) -> float:
    """获取用户简书钻数量

    # ! 当用户没有文章时，该函数将抛出 APIError 异常

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Raises:
        APIError: 由于用户没有文章导致无法获取总资产信息时抛出此异常

    Returns:
        float: 用户简书钻数量
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["jsd_balance"] / 1000
    if json_obj["total_wordage"] == 0 and result == 0:
        raise APIError("受 API 限制（用户无文章时无法获取总资产信息），无法获取该用户的简书钻数量")
    return result


def GetUserFTNCount(user_url: str, disable_check: bool = False) -> float:
    """获取用户简书贝数量

    # ! 视用户资产配置情况不同，该函数获取到的数值会有不大于 1000 的偏差

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        float: 用户简书贝数量
    """
    assets = GetUserAssetsCount(user_url)
    FTN = GetUserFPCount(user_url)
    result = assets - FTN
    result = abs(result)  # 处理用户简书贝数量较少导致结果为负的情况
    result = round(result, 3)  # 处理浮点数精度问题
    return result


def GetUserBadgesList(user_url: str, disable_check: bool = False) -> List[str]:
    """获取用户徽章列表

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[str]: 用户徽章列表
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    html_obj = GetUserPCHtmlDataApi(user_url)
    result = html_obj.xpath("//li[@class='badge-icon']/a/text()")
    result = [item.replace(" ", "").replace("\n", "") for item in result]  # 移除空格和换行符
    result = [item for item in result if item != ""]  # 去除空值
    return result


def GetUserLastUpdateTime(user_url: str, disable_check: bool = False) -> datetime:
    """获取用户文章最后更新时间

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        datetime: 用户文章最后更新时间
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result


def GetUserVIPInfo(user_url: str, disable_check: bool = False) -> Dict:
    """获取用户会员信息

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        Dict: 用户会员信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    try:
        result = {
            "vip_type": {
                "bronze": "铜牌",
                "silver": "银牌",
                "gold": "黄金",
                "platina": "白金"
            }[json_obj["member"]["type"]],
            "expire_date": datetime.fromtimestamp(json_obj["member"]["expires_at"])
        }
    except KeyError:
        result = {
            "vip_type": None,
            "expire_date": None
        }
    return result


def GetUserIntroductionHtml(user_url: str, disable_check: bool = False) -> str:
    """获取 Html 格式的用户简介

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: Html 格式的用户个人简介
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["intro"]
    return result


def GetUserIntroductionText(user_url: str, disable_check: bool = False) -> str:
    """获取纯文本格式的用户简介

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        str: 纯文本格式的用户个人简介
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    if json_obj["intro"] == "":   # 简介为空
        return ""
    html_obj = etree.HTML(json_obj["intro"])
    result = html_obj.xpath("//*/text()")
    result = "\n".join(result)
    return result


def GetUserNextAnniversaryDay(user_url: str, disable_check: bool = False) -> datetime:
    """获取用户的下一次简书周年纪念日

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        datetime: 用户的下一次简书周年纪念日
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    user_slug = UserUrlToUserSlug(user_url)
    html_obj = GetUserNextAnniversaryDayHtmlDataApi(user_slug)
    result = html_obj.xpath('//*[@id="app"]/div[1]/div/text()')[0]
    result = findall(r"\d+", result)
    result = datetime.fromisoformat("-".join(result))
    return result


def GetUserNotebooksInfo(user_url: str, disable_check: bool = False) -> List[Dict]:
    """获取用户文集与连载信息

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 用户文集与连载信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserCollectionsAndNotebooksJsonDataApi(user_url=user_url, user_slug=UserUrlToUserSlug(user_url))
    result = []
    for item in json_obj["notebooks"]:
        item_data = {
            "nid": item["id"],
            "name": item["name"],
            "is_book": item["book"]
        }
        if item["book"]:
            item_data["is_paid_book"] = item["paid_book"]  # 如果是连载，则判断是否是付费连载
        result.append(item_data)
    return result


def GetUserOwnCollectionsInfo(user_url: str, disable_check: bool = False) -> List[Dict]:
    """获取用户自己创建的专题信息

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 用户自己创建的专题信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserCollectionsAndNotebooksJsonDataApi(user_url=user_url, user_slug=UserUrlToUserSlug(user_url))
    result = []
    for item in json_obj["own_collections"]:
        item_data = {
            "cid": item["id"],
            "cslug": item["slug"],
            "name": item["title"],
            "avatar_url": item["avatar"]
        }
        result.append(item_data)
    return result


def GetUserManageableCollectionsInfo(user_url: str, disable_check: bool = False) -> List[Dict]:
    """获取用户管理的专题信息

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 用户管理的专题信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    json_obj = GetUserCollectionsAndNotebooksJsonDataApi(user_url=user_url, user_slug=UserUrlToUserSlug(user_url))
    result = []
    for item in json_obj["manageable_collections"]:
        item_data = {
            "cid": item["id"],
            "cslug": item["slug"],
            "name": item["title"],
            "avatar_url": item["avatar"]
        }
        result.append(item_data)
    return result


def GetUserArticlesInfo(user_url: str, page: int = 1, count: int = 10,
                        sorting_method: str = "time", disable_check: bool = False) -> List[Dict]:
    """获取用户文章信息

    Args:
        user_url (str): 用户个人主页 Url
        page (int, optional): 页码，与网页端文章顺序相同. Defaults to 1.
        count (int, optional): 获取的文章数量. Defaults to 10.
        sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 用户文章信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    order_by = {
        "time": "added_at",
        "comment_time": "commented_at",
        "hot": "top"
    }[sorting_method]
    json_obj = GetUserArticlesListJsonDataApi(user_url=user_url, page=page,
                                              count=count, order_by=order_by)
    result = []
    for item in json_obj:
        item_data = {
            "aid": item["object"]["data"]["id"],
            "title": item["object"]["data"]["title"],
            "aslug": item["object"]["data"]["slug"],
            "release_time": datetime.fromisoformat(item["object"]["data"]["first_shared_at"]),
            "first_image_url": item["object"]["data"]["list_image_url"],
            "summary": item["object"]["data"]["public_abbr"],
            "views_count": item["object"]["data"]["views_count"],
            "likes_count": item["object"]["data"]["likes_count"],
            "is_top": item["object"]["data"]["is_top"],
            "paid": item["object"]["data"]["paid"],
            "commentable": item["object"]["data"]["commentable"],
            "user": {
                "uid": item["object"]["data"]["user"]["id"],
                "name": item["object"]["data"]["user"]["nickname"],
                "uslug": item["object"]["data"]["user"]["slug"],
                "avatar_url": item["object"]["data"]["user"]["avatar"]
            },
            "total_fp_amount": item["object"]["data"]["total_fp_amount"] / 1000,
            "comments_count": item["object"]["data"]["public_comments_count"],
            "rewards_count": item["object"]["data"]["total_rewards_count"]
        }
        result.append(item_data)
    return result


def GetUserFollowingInfo(user_url: str, disable_check: bool = False, page: int = 1) -> List[Dict]:
    """获取用户关注者信息

    Args:
        user_url (str): 用户个人主页 Url
        page (int, optional): 关注列表页码. Defaults to 1.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 用户关注者信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    html_obj = GetUserFollowingListHtmlDataApi(user_url=user_url, page=page)
    name_raw_data = html_obj.xpath("//a[@class='name']")[1:]
    if not name_raw_data:  # 判断该页数据是否为空
        return []
    followers_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[1]")
    fans_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[2]")
    articles_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[3]")
    words_and_likes_raw_data = html_obj.xpath("//div[@class='meta'][2]")
    result = []
    for index in range(8):
        item_data = {
            "name": name_raw_data[index].text,
            "followers_count": int(followers_raw_data[index].text.replace("关注 ", "")),
            "fans_count": int(fans_raw_data[index].text.replace("粉丝", "")),
            "articles_count": int(articles_raw_data[index].text.replace("文章 ", "")),
            "words_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[0]),   # TODO: 重复运行正则匹配，影响效率，需要优化
            "likes_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[1])
        }
        result.append(item_data)
    return result


def GetUserFansInfo(user_url: str, page: int = 1, disable_check: bool = False) -> List[Dict]:
    """获取用户粉丝信息

    Args:
        user_url (str): 用户个人主页 Url
        page (int, optional): 粉丝列表页码. Defaults to 1.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 用户粉丝信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    html_obj = GetUserFollowersListHtmlDataApi(user_url=user_url, page=page)
    name_raw_data = html_obj.xpath("//a[@class='name']")[1:]
    if not name_raw_data:  # 判断该页数据是否为空
        return []
    followers_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[1]")
    fans_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[2]")
    articles_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[3]")
    words_and_likes_raw_data = html_obj.xpath("//div[@class='meta'][2]")
    result = []
    for index in range(8):
        item_data = {
            "name": name_raw_data[index].text,
            "followers_count": int(followers_raw_data[index].text.replace("关注 ", "")),
            "fans_count": int(fans_raw_data[index].text.replace("粉丝", "")),
            "articles_count": int(articles_raw_data[index].text.replace("文章 ", "")),
            "words_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[0]),   # TODO: 重复运行正则匹配，影响效率，需要优化
            "likes_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[1])
        }
        result.append(item_data)
    return result


def GetUserAllBasicData(user_url: str, disable_check: bool = False) -> Dict:
    """获取用户的所有基础信息

    Args:
        user_url (str): 用户个人主页 Url
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        Dict: 用户基础信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    result = {}
    json_obj = GetUserJsonDataApi(user_url)
    html_obj = GetUserPCHtmlDataApi(user_url)
    anniversary_day_html_obj = GetUserNextAnniversaryDayHtmlDataApi(UserUrlToUserSlug(user_url))

    result["name"] = json_obj["nickname"]
    result["url"] = user_url
    result["uslug"] = UserUrlToUserSlug(user_url)
    result["gender"] = json_obj["gender"]
    result["followers_count"] = json_obj["following_users_count"]
    result["fans_count"] = json_obj["followers_count"]
    result["articles_count"] = json_obj
    result["wordage"] = json_obj["total_wordage"]
    result["likes_count"] = json_obj["total_likes_count"]
    try:
        result["assets_count"] = html_obj.xpath("//div[@class='info']/ul/li[6]/div[@class='meta-block']/p")[0].text
        result["assets_count"] = float(result["assets_count"].replace(".", "").replace("w", "000"))
    except IndexError:
        result["assets_count"] = None
    if json_obj["total_wordage"] == 0 and json_obj["jsd_balance"] == 0:
        result["FP_count"] = None
    else:
        result["FP_count"] = json_obj["jsd_balance"] / 1000
    if result["assets_count"] and result["FP_count"]:
        result["FTN_count"] = result["assets_count"] - result["FP_count"]
        result["FTN_count"] = round(abs(result["FTN_count"]), 3)
    else:
        result["FTN_count"] = None
    result["badges_list"] = html_obj.xpath("//li[@class='badge-icon']/a/text()")
    result["badges_list"] = [item.replace(" ", "").replace("\n", "") for item in result["badges_list"]]  # 移除空格和换行符
    result["badges_list"] = [item for item in result["badges_list"] if item != ""]  # 去除空值
    result["last_update_time"] = datetime.fromtimestamp(json_obj["last_updated_at"])
    try:
        result["vip_info"] = {
            "vip_type": {
                    "bronze": "铜牌",
                    "silver": "银牌",
                    "gold": "黄金",
                    "platina": "白金"
                }[json_obj["member"]["type"]],
            "expire_date": datetime.fromtimestamp(json_obj["member"]["expires_at"])
        }
    except KeyError:
        result["vip_info"] = {
            "vip_type": None,
            "expire_date": None
        }
    result["introduction_html"] = json_obj["intro"]
    if not result["introduction_html"]:
        result["introduction_text"] = ""
    else:
        result["introduction_text"] = "\n".join(etree.HTML(result["introduction_html"]).xpath("//*/text()"))
    result["next_anniversary_day"] = anniversary_day_html_obj.xpath('//*[@id="app"]/div[1]/div/text()')[0]
    result["next_anniversary_day"] = datetime.fromisoformat("-".join(findall(r"\d+", result["next_anniversary_day"])))
    return result


def GetUserTimelineInfo(user_url: str, disable_check: bool = False, max_id: int = 1000000000) -> List[Dict]:
    """获取用户动态信息

    ！在极少数情况下可能会遇到不在可解析列表中的动态类型，此时程序会跳过这条动态，不会抛出异常

    Args:
        user_url (str): 用户个人主页 Url
        max_id (int, optional): 最大 id，值等于上一次获取到的数据中最后一项的 operation_id. Defaults to 1000000000.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Returns:
        List[Dict]: 用户动态信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    user_slug = UserUrlToUserSlug(user_url)
    html_obj = GetUserTimelineHtmlDataApi(user_slug, max_id)
    blocks = [x.__copy__() for x in html_obj.xpath("//li[starts-with(@id, 'feed-')]")]
    result = []

    for block in blocks:
        item_data = {}
        item_data["operation_id"] = int(block.xpath("//li/@id")[0][5:])
        item_data["operation_type"] = block.xpath("//span[starts-with(@data-datetime, '20')]/@data-type")[0]
        item_data["operation_time"] = datetime.fromisoformat(block.xpath("//span[starts-with(@data-datetime, '20')]/@data-datetime")[0])

        if item_data["operation_type"] == "like_note":  # 对文章点赞
            item_data["operation_type"] = "like_article"  # 鬼知道谁把对文章点赞写成 like_note 的
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][3:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["target_article_title"] = block.xpath("//a[@class='title']/text()")[0]
            item_data["target_article_url"] = ArticleSlugToArticleUrl(block.xpath("//a[@class='title']/@href")[0][3:])
            item_data["target_user_name"] = block.xpath("//div[@class='origin-author']/a/text()")[0]
            item_data["target_user_url"] = UserSlugToUserUrl(block.xpath("//div[@class='meta']/a/@href")[0][3:])
            item_data["target_article_reads_count"] = int(block.xpath("//div[@class='meta']/a/text()")[1])
            item_data["target_article_likes_count"] = int(block.xpath("//div[@class='meta']/span/text()")[0])
            try:
                item_data["target_article_comments_count"] = int(block.xpath("//div[@class='meta']/a/text()")[3])
            except IndexError:  # 文章没有评论或评论区关闭
                item_data["target_article_comments_count"] = 0
            try:
                item_data["target_article_description"] = block.xpath("//p[@class='abstract']/text()")[0]
            except IndexError:  # 文章没有摘要
                item_data["target_article_description"] = ""

        elif item_data["operation_type"] == "like_comment":  # 对评论点赞
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][3:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["comment_content"] = "\n".join(block.xpath("//p[@class='comment']/text()"))
            item_data["target_article_title"] = block.xpath("//blockquote/div/span/a/text()")[0]
            item_data["target_article_url"] = ArticleSlugToArticleUrl(block.xpath("//blockquote/div/span/a/@href")[0][3:])
            item_data["target_user_name"] = block.xpath("//blockquote/div/a/text()")[0]
            item_data["target_user_url"] = UserSlugToUserUrl(block.xpath("//blockquote/div/a/@href")[0][3:])

        elif item_data["operation_type"] == "share_note":  # 发表文章
            item_data["operation_type"] = "publish_article"  # 鬼知道谁把发表文章写成 share_note 的
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][3:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["target_article_title"] = block.xpath("//a[@class='title']/text()")[0]
            item_data["target_article_url"] = ArticleSlugToArticleUrl(block.xpath("//a[@class='title']/@href")[0][3:])
            item_data["target_article_reads_count"] = int(block.xpath("//div[@class='meta']/a/text()")[1])
            item_data["target_article_likes_count"] = int(block.xpath("//div[@class='meta']/span/text()")[0])
            item_data["target_article_description"] = "\n".join(block.xpath("//p[@class='abstract']/text()"))
            try:
                item_data["target_article_comments_count"] = int(block.xpath("//div[@class='meta']/a/text()")[3])
            except IndexError:
                item_data["target_article_comments_count"] = 0

        elif item_data["operation_type"] == "comment_note":  # 发表评论
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][3:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["comment_content"] = "\n".join(block.xpath("//p[@class='comment']/text()"))
            item_data["target_article_title"] = block.xpath("//a[@class='title']/text()")[0]
            item_data["target_article_url"] = ArticleSlugToArticleUrl(block.xpath("//a[@class='title']/@href")[0][3:])
            item_data["target_user_name"] = block.xpath("//div[@class='origin-author']/a/text()")[0]
            item_data["target_user_url"] = UserSlugToUserUrl(block.xpath("//div[@class='meta']/a/@href")[0][3:])
            item_data["target_article_reads_count"] = int(block.xpath("//div[@class='meta']/a/text()")[1])
            item_data["target_article_likes_count"] = int(block.xpath("//div[@class='meta']/span/text()")[0])
            try:
                item_data["target_article_comments_count"] = int(block.xpath("//div[@class='meta']/a/text()")[3])
            except IndexError:  # 文章没有评论或评论区关闭
                item_data["target_article_comments_count"] = 0
            try:
                item_data["target_article_description"] = block.xpath("//p[@class='abstract']/text()")[0]
            except IndexError:  # 文章没有描述
                item_data["target_article_description"] = ""
            try:
                item_data["target_article_rewards_count"] = int(block.xpath("//div[@class='meta']/span/text()")[1])
            except IndexError:  # 没有赞赏数据
                item_data["target_article_rewards_count"] = 0

        elif item_data["operation_type"] == "like_notebook":  # 关注文集
            item_data["operation_type"] = "follow_notebook"  # 鬼知道谁把关注文集写成 like_notebook 的
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][4:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["target_notebook_title"] = block.xpath("//a[@class='title']/text()")[0]
            item_data["target_notebook_url"] = NotebookSlugToNotebookUrl(block.xpath("//a[@class='title']/@href")[0][3:])
            item_data["target_notebook_avatar_url"] = block.xpath("//div[@class='follow-detail']/div/a/img/@src")[0]
            item_data["target_user_name"] = block.xpath("//a[@class='creater']/text()")[0]
            item_data["target_user_url"] = UserSlugToUserUrl(block.xpath("//a[@class='creater']/@href")[0][3:])
            item_data["target_notebook_articles_count"] = int(findall(r"\d+", block.xpath("//div[@class='info'][1]/p/text()")[1])[0])
            item_data["target_notebook_subscribers_count"] = int(findall(r"\d+", block.xpath("//div[@class='info'][1]/p/text()")[1])[1])

        elif item_data["operation_type"] == "like_collection":  # 关注专题
            item_data["operator_type"] = "follow_collection"  # 鬼知道谁把关注专题写成 like_collection 的
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][4:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["target_collecton_title"] = block.xpath("//a[@class='title']/text()")[0]
            item_data["target_collecton_url"] = CollectionSlugToCollectionUrl(block.xpath("//a[@class='title']/@href")[0][3:])
            item_data["target_collecton_avatar_url"] = block.xpath("//div[@class='follow-detail']/div/a/img/@src")[0]
            item_data["target_user_name"] = block.xpath("//a[@class='creater']/text()")[0]
            item_data["target_user_url"] = UserSlugToUserUrl(block.xpath("//a[@class='creater']/@href")[0][3:])
            item_data["target_collecton_articles_count"] = int(findall(r"\d+", block.xpath("//div[@class='info'][1]/p/text()")[1])[0])
            item_data["target_collecton_subscribers_count"] = int(findall(r"\d+", block.xpath("//div[@class='info'][1]/p/text()")[1])[1])

        elif item_data["operation_type"] == "like_user":  # 关注用户
            item_data["operation_type"] = "follow_user"  # 鬼知道谁把关注用户写成 like_user 的
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][4:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["target_user_name"] = block.xpath("//div[@class='info']/a[@class='title']/text()")[0]
            item_data["target_user_url"] = UserSlugToUserUrl(block.xpath("//div[@class='info']/a[@class='title']/@href")[0][3:])
            item_data["target_user_wordage"] = int(findall(r"\d+", block.xpath("//div[@class='follow-detail']/div[@class='info']/p/text()")[0])[0])
            item_data["target_user_fans_count"] = int(findall(r"\d+", block.xpath("//div[@class='follow-detail']/div[@class='info']/p/text()")[0])[1])
            item_data["target_user_likes_count"] = int(findall(r"\d+", block.xpath("//div[@class='follow-detail']/div[@class='info']/p/text()")[0])[2])
            item_data["target_user_description"] = "\n".join(block.xpath("//div[@class='signature']/text()"))

        elif item_data["operation_type"] == "reward_note":  # 赞赏文章
            item_data["operation_type"] = "reward_article"  # 鬼知道谁把赞赏文章写成 reward_note 的
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][4:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]
            item_data["target_article_title"] = block.xpath("//a[@class='title']/text()")[0]
            item_data["target_article_url"] = ArticleSlugToArticleUrl(block.xpath("//a[@class='title']/@href")[0][3:])
            item_data["target_user_name"] = block.xpath("//div[@class='origin-author']/a/text()")[0]
            item_data["target_user_url"] = UserSlugToUserUrl(block.xpath("//div[@class='meta']/a/@href")[0][3:])
            item_data["target_article_reads_count"] = int(block.xpath("//div[@class='meta']/a/text()")[1])
            item_data["target_article_likes_count"] = int(block.xpath("//div[@class='meta']/span/text()")[0])
            try:
                item_data["target_article_comments_count"] = int(block.xpath("//div[@class='meta']/a/text()")[3])
            except IndexError:  # 文章没有评论或评论区关闭
                item_data["target_article_comments_count"] = 0
            try:
                item_data["target_article_description"] = block.xpath("//p[@class='abstract']/text()")[0]
            except IndexError:  # 文章没有描述
                item_data["target_article_description"] = ""
            try:
                item_data["target_article_rewards_count"] = int(block.xpath("//div[@class='meta']/span/text()")[1])
            except IndexError:  # 没有赞赏数据
                item_data["target_article_rewards_count"] = 0

        elif item_data["operation_type"] == "join_jianshu":  # 加入简书
            item_data["operator_name"] = block.xpath("//a[@class='nickname']/text()")[0]
            item_data["operator_url"] = UserSlugToUserUrl(block.xpath("//a[@class='nickname']/@href")[0][4:])
            item_data["operator_avatar_url"] = block.xpath("//a[@class='avatar']/img/@src")[0]

        result.append(item_data)
    return result


def GetUserAllArticlesInfo(user_url: str, count: int = 10, sorting_method: str = "time",
                           max_count: int = None, disable_check: bool = False) -> Generator[Dict, None, None]:
    """获取用户的所有文章信息

    Args:
        user_url (str): 用户个人主页 Url
        count (int, optional): 单次获取的数据数量，会影响性能. Defaults to 10.
        sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".
        max_count (int, optional): 获取的文章信息数量上限，Defaults to None.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Yields:
        Iterator[Dict], None, None]: 文章信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    page = 1
    now_count = 0
    while True:
        result = GetUserArticlesInfo(user_url, page, count, sorting_method, disable_check=True)
        if result:
            page += 1
        else:  # 没有新的数据
            return
        for item in result:
            yield item
            if max_count:  # 如果有上限
                now_count += 1
                if now_count == max_count:  # 达到上限
                    return


def GetUserAllFollowingInfo(user_url: str, max_count: int = None, disable_check: bool = False) -> Generator[Dict, None, None]:
    """获取用户的所有关注者信息

    Args:
        user_url (str): 用户个人主页 Url
        max_count (int, optional): 获取的关注者信息数量上限，Defaults to None.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Yields:
        Iterator[Dict], None, None]: 关注者信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    page = 1
    now_count = 0
    while True:
        result = GetUserFollowingInfo(user_url, page, disable_check=True)
        if result:
            page += 1
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if now_count == max_count:
                    return


def GetUserAllFansInfo(user_url: str, max_count: int = None, disable_check: bool = False) -> Generator[Dict, None, None]:
    """获取用户的所有粉丝信息

    Args:
        user_url (str): 用户个人主页 Url
        max_count (int, optional): 获取的粉丝信息数量上限，Defaults to None.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Yields:
        Iterator[Dict], None, None]: 粉丝信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    page = 1
    now_count = 0
    while True:
        result = GetUserFansInfo(user_url, page, disable_check=True)
        if result:
            page += 1
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if now_count == max_count:
                    return


def GetUserAllTimelineInfo(user_url: str, max_count: int = None, disable_check: bool = False) -> Generator[Dict, None, None]:
    """获取用户的所有动态信息

    Args:
        user_url (str): 用户个人主页 Url
        max_count (int, optional): 获取的动态信息数量上限，Defaults to None.
        disable_check (bool): 禁用参数有效性检查. Defaults to False.

    Yields:
        Iterator[Dict], None, None]: 动态信息
    """
    if not disable_check:
        AssertUserUrl(user_url)
        AssertUserStatusNormal(user_url)
    max_id = None
    now_count = 0
    while True:
        result = GetUserTimelineInfo(user_url, max_id, disable_check=True)
        if result:
            max_id = result[-1]["operation_id"]
        else:
            return
        for item in result:
            yield item
            if max_count:
                now_count += 1
                if now_count == max_count:
                    return



from datetime import datetime
from typing import Dict, List

from . import article, collection, island, notebook, user
from .assert_funcs import (AssertArticleUrl, AssertCollectionUrl,
                           AssertIslandUrl, AssertNotebookUrl, AssertUserUrl)
from .convert import (ArticleSlugToArticleUrl, CollectionSlugToCollectionUrl,
                      IslandSlugToIslandUrl, IslandUrlToIslandSlug,
                      NotebookSlugToNotebookUrl, UserSlugToUserUrl,
                      UserUrlToUserSlug)
from .exceptions import InputError

DISABLE_CACHE = False  # 禁用缓存


def cache_result(func):
    """缓存函数的返回值"""
    cache_Dict = {}

    def wrapper(*args, **kwargs):
        args_hash = hash(tuple(args[1:]) + tuple(kwargs.items()))
        cache_result = cache_Dict.get(args_hash)
        if cache_result and not DISABLE_CACHE:
            return cache_result
        else:
            result = func(*args, **kwargs)
            if not DISABLE_CACHE:
                cache_Dict[args_hash] = result
            return result
    return wrapper


class User():
    """用户类
    """
    def __init__(self, user_url: str = None, *, user_slug: str = None):
        """构建新的用户对象

        Args:
            user_url (str, optional): 用户个人主页 Url. Defaults to None.
            user_slug (str, optional): 用户 Slug. Defaults to None.
        """
        # TODO: 支持使用用户 Id 初始化用户对象
        if user_slug and user_url:
            raise InputError("只能使用一个参数进行用户类的初始化")
        elif user_url:
            AssertUserUrl(user_url)
            self._url = user_url
        elif user_slug:
            user_url = UserSlugToUserUrl(user_slug)
            AssertUserUrl(user_url)
            self._url = user_url
        else:
            raise InputError("请至少传入一个参数")

    @property
    def url(self) -> str:
        """获取用户主页 Url

        Returns:
            str: 用户主页 Url
        """
        return self._url

    @property
    @cache_result
    def slug(self) -> str:
        """获取用户 Slug

        Returns:
            str: 用户 Slug
        """
        return UserUrlToUserSlug(self._url)

    @property
    @cache_result
    def name(self) -> str:
        """获取用户昵称

        Returns:
            str: 用户昵称
        """
        return user.GetUserName(self._url, disable_check=True)

    @property
    @cache_result
    def gender(self) -> int:
        """获取用户性别

        Returns:
            int: 用户性别，0 为未知，1 为男，2 为女
        """
        return user.GetUserGender(self._url)

    @property
    @cache_result
    def followers_count(self) -> int:
        """获取用户关注数

        Returns:
            int: 关注数
        """
        return user.GetUserFollowersCount(self._url, disable_check=True)

    @property
    @cache_result
    def fans_count(self) -> int:
        """获取用户粉丝数

        Returns:
            int: 粉丝数
        """
        return user.GetUserFansCount(self._url, disable_check=True)

    @property
    @cache_result
    def articles_count(self) -> int:
        """获取用户文章数

        Returns:
            int: 文章数
        """
        return user.GetUserArticlesCount(self._url, disable_check=True)

    @property
    @cache_result
    def wordage(self) -> int:
        """获取用户总字数

        Returns:
            int: 总字数
        """
        return user.GetUserWordage(self._url, disable_check=True)

    @property
    @cache_result
    def likes_count(self) -> int:
        """获取用户被点赞数

        Returns:
            int: 被点赞数
        """
        return user.GetUserLikesCount(self._url, disable_check=True)

    @property
    @cache_result
    def assets_count(self) -> float:
        """获取用户资产量

        Returns:
            int: 资产量
        """
        return user.GetUserAssetsCount(self._url, disable_check=True)

    @property
    @cache_result
    def FP_count(self) -> float:
        """获取用户简书钻数量

        Returns:
            int: 简书钻数量
        """
        return user.GetUserFPCount(self._url, disable_check=True)

    @property
    @cache_result
    def FTN_count(self) -> float:
        """获取用户简书贝数量

        Returns:
            int: 简书贝数量
        """
        return user.GetUserFTNCount(self._url, disable_check=True)

    @property
    @cache_result
    def badges(self) -> List:
        """获取徽章列表

        Returns:
            List: 徽章列表
        """
        return user.GetUserBadgesList(self._url, disable_check=True)

    @property
    @cache_result
    def last_update_time(self) -> datetime:
        """获取最近更新时间

        Returns:
            datetime: 最近更新时间
        """
        return user.GetUserLastUpdateTime(self._url, disable_check=True)

    @property
    @cache_result
    def VIP_info(self) -> Dict:
        """获取用户会员信息

        Returns:
            Dict: 会员信息
        """
        return user.GetUserVIPInfo(self._url, disable_check=True)

    @property
    @cache_result
    def introduction_text(self) -> str:
        """获取纯文本格式的用户简介

        Returns:
            str: 纯文本格式的用户简介
        """
        return user.GetUserIntroductionText(self._url, disable_check=True)

    @property
    @cache_result
    def introduction_html(self) -> str:
        """获取 Html 格式的用户简介

        Returns:
            str: Html 格式的用户简介
        """
        return user.GetUserIntroductionHtml(self._url, disable_check=True)

    @property
    @cache_result
    def notebooks(self) -> List:
        """获取用户文集信息

        Returns:
            List: 文集信息
        """
        return user.GetUserNotebooksInfo(self._url, disable_check=True)

    @property
    @cache_result
    def own_collections(self) -> List:
        """获取自己创建的专题信息

        Returns:
            List: 自己创建的专题信息
        """
        return user.GetUserOwnCollectionsInfo(self._url, disable_check=True)

    @property
    @cache_result
    def manageable_collections(self) -> List:
        """获取用户有管理权的专题信息

        Returns:
            List: 有管理权的专题信息
        """
        return user.GetUserManageableCollectionsInfo(self._url, disable_check=True)

    @cache_result
    def articles_info(self, page: int = 1, count: int = 10) -> List[Dict]:
        """获取文章信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            count (int, optional): 每次获取的文章信息数量. Defaults to 1.

        Returns:
            List[Dict]: 文章信息
        """
        return user.GetUserArticlesInfo(self._url, page, count, disable_check=True)

    @cache_result
    def followers_info(self, page: int = 1) -> List[Dict]:
        """获取关注者信息

        Args:
            page (int, optional): 页码. Defaults to 1.

        Returns:
            List[Dict]: 关注者信息
        """
        return user.GetUserFollowersInfo(self._url, page, disable_check=True)

    @cache_result
    def fans_info(self, page: int = 1) -> List[Dict]:
        """获取粉丝信息

        Args:
            page (int, optional): 页码. Defaults to 1.

        Returns:
            List[Dict]: 粉丝信息
        """
        return user.GetUserFansInfo(self._url, page, disable_check=True)

    def __eq__(self, other: object) -> bool:
        """判断是否是同一个用户

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if not isinstance(other, User):
            return False  # 不是由用户类构建的必定不相等
        if self._url == other._url:
            return True
        else:
            return False

    def __str__(self) -> str:
        """输出用户信息摘要

        Returns:
            str: 用户信息摘要
        """
        result = f"""用户信息摘要：
昵称：{self.name}
Url：{self.url}
性别：{self.gender}
关注者数：{self.followers_count}
粉丝数：{self.fans_count}
文章数：{self.articles_count}
总字数：{self.wordage}
简书钻：{self.FP_count}
简书贝：{self.FTN_count}
总资产：{self.assets_count}
徽章：{' '.join(self.badges)}
最后更新时间：{self.last_update_time}
会员等级：{self.VIP_info['vip_type']}
会员过期时间：{self.VIP_info['expire_date']}
个人简介：\n{self.introduction_text}"""
        return result


class Article():
    """文章类
    """
    def __init__(self, article_url: str = None, article_slug: str = None):
        """构建新的文章对象

        Args:
            article_url (str, optional): 文章 Url. Defaults to None.
            article_slug (str, optional): 文章 Slug. Defaults to None.
        """
        # TODO: 支持使用文章 Id 初始化文章对象
        if article_slug and article_url:
            raise InputError("只能使用一个参数进行文章类的初始化")
        elif article_url:
            AssertArticleUrl(article_url)
            self._url = article_url
        elif article_slug:
            article_url = ArticleSlugToArticleUrl(article_slug)
            AssertArticleUrl(article_url)
            self._url = article_url
        else:
            raise InputError("请至少传入一个参数")

    @property
    def url(self) -> str:
        """获取文章 Url

        Returns:
            str: 文章 Url
        """
        return self._url

    @property
    @cache_result
    def slug(self) -> str:
        """获取文章 Slug

        Returns:
            str: 文章 Slug
        """
        return article.GetArticleSlug(self._url)

    @property
    @cache_result
    def title(self) -> str:
        """获取文章标题

        Returns:
            str: 标题
        """
        return article.GetArticleTitle(self._url, disable_check=True)

    @property
    @cache_result
    def author_name(self) -> str:
        """获取文章作者名

        Returns:
            str: 作者名
        """
        return article.GetArticleAuthorName(self._url, disable_check=True)

    @property
    @cache_result
    def wordage(self) -> int:
        """获取文章总字数

        Returns:
            int: 总字数
        """
        return article.GetArticleWordage(self._url, disable_check=True)

    @property
    @cache_result
    def reads_count(self) -> int:
        """获取文章阅读量

        Returns:
            int: 阅读量
        """
        return article.GetArticleReadsCount(self._url, disable_check=True)

    @property
    @cache_result
    def likes_count(self) -> int:
        """获取文章点赞量

        Returns:
            int: 文章点赞量
        """
        return article.GetArticleLikesCount(self._url, disable_check=True)

    @property
    @cache_result
    def comments_count(self) -> int:
        """获取文章评论量

        Returns:
            int: 文章评论量
        """
        return article.GetArticleCommentsCount(self._url, disable_check=True)

    @property
    @cache_result
    def most_valuable_comments_count(self) -> int:
        """获取文章精选评论量

        Returns:
            int: 文章精选评论量
        """
        return article.GetArticleMostValuableCommentsCount(self._url, disable_check=True)

    @property
    @cache_result
    def total_FP_count(self) -> float:
        """获取文章总获钻量

        Returns:
            int: 文章总获钻量
        """
        return article.GetArticleTotalFPCount(self._url, disable_check=True)

    @property
    @cache_result
    def description(self) -> str:
        """获取文章摘要

        Returns:
            str: 文章摘要
        """
        return article.GetArticleDescription(self._url, disable_check=True)

    @property
    @cache_result
    def publish_time(self) -> datetime:
        """获取文章发布时间

        Returns:
            datetime: 文章发布时间
        """
        return article.GetArticlePublishTime(self._url, disable_check=True)

    @property
    @cache_result
    def update_time(self) -> datetime:
        """获取文章更新时间

        Returns:
            datetime: 文章更新时间
        """
        return article.GetArticleUpdateTime(self._url, disable_check=True)

    @property
    @cache_result
    def paid_status(self) -> bool:
        """获取文章付费状态

        Returns:
            bool: 文章付费状态
        """
        return article.GetArticlePaidStatus(self._url, disable_check=True)

    @property
    @cache_result
    def reprint_status(self) -> bool:
        """获取文章转载状态

        Returns:
            bool: 文章转载状态
        """
        return article.GetArticleReprintStatus(self._url, disable_check=True)

    @property
    @cache_result
    def comment_status(self) -> bool:
        """获取文章评论状态

        Returns:
            bool: 文章评论状态
        """
        return article.GetArticleCommentStatus(self._url, disable_check=True)

    @property
    @cache_result
    def html(self) -> str:
        """获取 Html 格式的文章内容

        Returns:
            str: Html 格式的文章内容
        """
        return article.GetArticleHtml(self._url, disable_check=True)

    @property
    @cache_result
    def text(self) -> str:
        """获取纯文本格式的文章内容

        Returns:
            str: 纯文本格式的文章内容
        """
        return article.GetArticleText(self._url, disable_check=True)

    def __eq__(self, other: object) -> bool:
        """判断是否是同一篇文章

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if not isinstance(other, Article):
            return False  # 不是由文章类构建的必定不相等
        if self._url == other._url:
            return True
        else:
            return False

    def __str__(self) -> str:
        result = f"""文章信息摘要：
标题：{self.title}
Url: {self.url}
作者名：{self.author_name}
字数：{self.wordage}
阅读量：{self.reads_count}
点赞数：{self.likes_count}
评论数：{self.comments_count}
精选评论数：{self.most_valuable_comments_count}
总获钻量：{self.total_FP_count}
发布时间：{self.publish_time}
更新时间：{self.update_time}
需付费：{self.paid_status}
可转载：{self.reprint_status}
可评论：{self.comment_status}
摘要：\n{self.description}"""
        return result


class Notebook():
    """文集类
    """
    def __init__(self, notebook_url: str = None, notebook_slug: str = None):
        """构建新的文集对象

        Args:
            notebook_url (str, optional): 文集 Url. Defaults to None.
            notebook_slug (str, optional): 文集 Slug. Defaults to None.
        """
        # TODO: 支持使用用户 Id 初始化用户对象
        if notebook_slug and notebook_url:
            raise InputError("只能使用一个参数进行文集类的初始化")
        elif notebook_url:
            AssertNotebookUrl(notebook_url)
            self._url = notebook_url
        elif notebook_slug:
            notebook_url = NotebookSlugToNotebookUrl(notebook_slug)
            AssertNotebookUrl(notebook_url)
            self._url = notebook_url
        else:
            raise InputError("请至少传入一个参数")

    @property
    def url(self) -> str:
        """获取文集 Url

        Returns:
            str: 文集 Url
        """
        return self._url

    @property
    @cache_result
    def id(self) -> int:
        """获取文集 ID

        Returns:
            int: 文集 ID
        """
        return notebook.GetNotebookId(self._url)

    @property
    @cache_result
    def slug(self) -> str:
        """获取文集 Slug

        Returns:
            str: 文集 Slug
        """
        return notebook.GetNotebookSlug(self._url)

    @property
    @cache_result
    def name(self) -> str:
        """获取文集名称

        Returns:
            str: 文集名称
        """
        return notebook.GetNotebookName(self._url, disable_check=True)

    @property
    @cache_result
    def articles_count(self) -> int:
        """获取文集中的文章总数

        Returns:
            int: 文章总数
        """
        return notebook.GetNotebookArticlesCount(self._url, disable_check=True)

    @property
    @cache_result
    def author_name(self) -> str:
        """获取文集的作者名

        Returns:
            str: 作者名
        """
        return notebook.GetNotebookAuthorInfo(self._url, disable_check=True)["name"]

    @property
    @cache_result
    def author_info(self) -> Dict:
        """获取文集的作者信息

        Returns:
            Dict: 作者信息
        """
        return notebook.GetNotebookAuthorInfo(self._url, disable_check=True)

    @property
    @cache_result
    def wordage(self) -> int:
        """获取文集中所有文章的总字数

        Returns:
            int: 文集总字数
        """
        return notebook.GetNotebookWordage(self._url, disable_check=True)

    @property
    @cache_result
    def subscribers_count(self) -> int:
        """获取文集的关注者数量

        Returns:
            int: 关注者数量
        """
        return notebook.GetNotebookSubscribersCount(self._url, disable_check=True)

    @property
    @cache_result
    def update_time(self) -> datetime:
        """获取文集的更新时间

        Returns:
            datetime: 更新时间
        """
        return notebook.GetNotebookUpdateTime(self._url, disable_check=True)

    @cache_result
    def articles_info(self, page: int = 1, count: int = 10, sorting_method: str = "time") -> List[Dict]:
        """获取文集中的文章信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            count (int, optional): 每次返回的数据数量. Defaults to 10.
            sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
            comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".

        Returns:
            List[Dict]: 文章信息
        """
        return notebook.GetNotebookArticlesInfo(self._url, page, count, sorting_method, disable_check=True)

    def __eq__(self, other: object) -> bool:
        """判断是否是同一个文集

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if not isinstance(other, Notebook):
            return False  # 不是由文集类构建的必定不相等
        if self._url == other._url:
            return True
        else:
            return False

    def __str__(self) -> str:
        """输出文集信息摘要

        Returns:
            str: 文集信息摘要
        """
        result = f"""文集信息摘要：
名称：{self.name}
Url：{self.url}
作者名：{self.author_name}
文章数：{self.articles_count}
总字数：{self.wordage}
关注者数：{self.subscribers_count}
更新时间：{self.update_time}"""

        return result


class Collection():
    """专题类
    """
    def __init__(self, collection_url: str = None, collection_slug: str = None,
                 collection_id: int = None):
        """初始化专题类

        Args:
            collection_url (str, optional): 专题 Url. Defaults to None.
            collection_slug (str, optional): 专题 Slug. Defaults to None.
            collection_id (int, optional): 专题 Id，如不传入部分数据将无法获取. Defaults to None.
        """
        # TODO: 支持通过 collection_url 获取 collection_id
        if collection_slug and collection_url:
            raise InputError("只能使用一个参数进行专题类的初始化")
        elif collection_url:
            AssertCollectionUrl(collection_url)
            self._url = collection_url
        elif collection_slug:
            collection_url = CollectionSlugToCollectionUrl(collection_slug)
            AssertCollectionUrl(collection_url)
            self._url = collection_url
        else:
            raise InputError("请至少传入一个参数")

        if collection_id:
            self._id = collection_id
        else:
            self._id = None

    @property
    def url(self) -> str:
        """获取专题 Url

        Returns:
            str: 专题 Url
        """
        return self._url

    @property
    @cache_result
    def slug(self) -> str:
        """获取专题 Slug

        Returns:
            str: 专题 Slug
        """
        return collection.GetCollectionSlug(self._url)

    @property
    @cache_result
    def name(self) -> str:
        """获取专题名称

        Returns:
            str: 专题名称
        """
        return collection.GetCollectionName(self._url, disable_check=True)

    @property
    @cache_result
    def avatar_url(self) -> str:
        """获取专题头像链接

        Returns:
            str: 专题头像链接
        """
        return collection.GetCollectionAvatarUrl(self._url, disable_check=True)

    @property
    @cache_result
    def introduction_text(self) -> str:
        """获取纯文本格式的专题简介

        Returns:
            str: 纯文本格式的专题简介
        """
        return collection.GetCollectionIntroductionText(self._url, disable_check=True)

    @property
    @cache_result
    def introduction_html(self) -> str:
        """获取 Html 格式的专题简介

        Returns:
            str:  Html 格式的专题简介
        """
        return collection.GetCollectionIntroductionHtml(self._url, disable_check=True)

    @property
    @cache_result
    def articles_update_time(self) -> datetime:
        """获取专题文章更新时间

        Returns:
            datetime: 专题文章更新时间
        """
        return collection.GetCollectionArticlesUpdateTime(self._url, disable_check=True)

    @property
    @cache_result
    def info_update_time(self) -> datetime:
        """获取专题信息更新时间

        Returns:
            datetime: 专题信息更新时间
        """
        return collection.GetCollectionInfoUpdateTime(self._url, disable_check=True)

    @property
    @cache_result
    def owner_info(self) -> Dict:
        """获取专题的所有者信息

        Returns:
            Dict: 用户信息
        """
        return collection.GetCollectionOwnerInfo(self._url, disable_check=True)

    @property
    @cache_result
    def articles_count(self) -> int:
        """获取专题文章数

        Returns:
            int: 专题文章数
        """
        return collection.GetCollectionArticlesCount(self._url, disable_check=True)

    @property
    @cache_result
    def subscribers_count(self) -> int:
        """获取专题关注者数

        Returns:
            int: 专题关注者数
        """
        return collection.GetCollectionSubscribersCount(self._url, disable_check=True)

    @cache_result
    def editors_info(self, page: int = 1) -> List[Dict]:
        """获取专题编辑信息

        Args:
            page (int, optional): 页码. Defause to 1.

        Raises:
            InputError: 因缺少 ID 参数而无法获取结果时抛出此异常

        Returns:
            List[Dict]: 编辑信息
        """
        if not self._id:
            raise InputError("实例化该专题对象时未传入 ID 参数，无法获取编辑信息")
        return collection.GetCollectionEditorsInfo(self._id, page)

    @cache_result
    def recommended_writers_info(self, page: int = False) -> List[Dict]:
        """获取专题推荐作者信息

        Args:
            page (int, optional): 页码. Defaults to False.

        Raises:
            InputError: 因缺少 ID 参数而无法获取结果时抛出此异常

        Returns:
            List[Dict]: 推荐作者信息
        """
        if not self._id:
            raise InputError("实例化该专题对象时未传入 ID 参数，无法获取推荐作者信息")
        return collection.GetCollectionRecommendedWritersInfo(self._id, page)

    @cache_result
    def subscribers_info(self, start_sort_id: int) -> List:
        """获取专题关注者信息

        Args:
            start_sort_id (int): 起始序号，等于上一条数据的序号

        Raises:
            InputError: 因缺少 ID 参数而无法获取结果时抛出此异常

        Returns:
            List: 关注者信息
        """
        if not self._id:
            raise InputError("实例化该专题对象时未传入 ID 参数，无法获取关注者信息")
        return collection.GetCollectionSubscribersInfo(self._id, start_sort_id)

    @cache_result
    def articles_info(self, page: int = 1, count: int = 10,
                      sorting_method: str = "time") -> List[Dict]:
        """获取专题文章信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            count (int, optional): 每次返回的数据数量. Defaults to 10.
            sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
            comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".

        Returns:
            List[Dict]: 专题中的文章信息
        """
        return collection.GetCollectionArticlesInfo(self._url, page, count, sorting_method, disable_check=True)

    def __eq__(self, other: object) -> bool:
        """判断是否是同一个专题

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if not isinstance(other, Collection):
            return False  # 不是由专题类构建的必定不相等
        if self._url == other._url:
            return True
        else:
            return False

    def __str__(self) -> str:
        """输出专题信息摘要

        Returns:
            str: 专题信息摘要
        """
        result = f"""
专题信息摘要：
专题名：{self.name}
Url：{self.url}
主编名：{self.owner_info["name"]}
图片链接：{self.avatar_url}
文章数：{self.articles_count}
关注者数：{self.subscribers_count}
文章更新时间：{self.articles_update_time}
信息更新时间：{self.info_update_time}
简介：\n{self.introduction_text}"""

        return result


class Island():
    """小岛类
    """
    def __init__(self, island_url: str = None, island_slug: str = None):
        if island_slug and island_url:
            raise InputError("只能使用一个参数进行小岛类的初始化")
        elif island_url:
            AssertIslandUrl(island_url)
            self._url = island_url
        elif island_slug:
            island_url = IslandSlugToIslandUrl(island_slug)
            AssertIslandUrl(island_url)
            self._url = island_url
        else:
            raise InputError("请至少传入一个参数")

    @property
    def url(self) -> str:
        """获取小岛 Url

        Returns:
            str: 小岛 Url
        """
        return self._url

    @property
    @cache_result
    def slug(self) -> str:
        """获取小岛 Slug

        Returns:
            str: 小岛 Slug
        """
        return IslandUrlToIslandSlug(self._url)

    @property
    @cache_result
    def name(self) -> str:
        """获取小岛名称

        Returns:
            str: 小岛名称
        """
        return island.GetIslandName(self._url, disable_check=True)

    @property
    @cache_result
    def avatar_url(self) -> str:
        """获取小岛头像链接

        Returns:
            str: 小岛头像链接
        """
        return island.GetIslandAvatarUrl(self._url, disable_check=True)

    @property
    @cache_result
    def introduction(self) -> str:
        """获取小岛简介

        Returns:
            str: 小岛简介
        """
        return island.GetIslandIntroduction(self._url, disable_check=True)

    @property
    @cache_result
    def members_count(self) -> int:
        """获取小岛成员数量

        Returns:
            int: 成员数量
        """
        return island.GetIslandMembersCount(self._url, disable_check=True)

    @property
    @cache_result
    def posts_count(self) -> int:
        """获取小岛帖子数量

        Returns:
            int: 帖子数量
        """
        return island.GetIslandPostsCount(self._url, disable_check=True)

    @property
    @cache_result
    def category(self) -> str:
        """获取小岛分类

        Returns:
            str: 分类
        """
        return island.GetIslandCategory(self._url, disable_check=True)

    @cache_result
    def posts(self, start_sort_id: int = None, count: int = 10,
              topic_id: int = None, sorting_method: str = "time") -> List[Dict]:
        """获取小岛帖子信息

        Args:
            start_sort_id (int, optional): 起始序号，等于上一条数据的序号. Defaults to None.
            count (int, optional): 每次返回的数据数量. Defaults to 10.
            topic_id (int, optional): 话题 ID. Defaults to None.
            sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
            comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".

        Returns:
            List[Dict]: 帖子信息
        """
        return island.GetIslandPosts(self._url, start_sort_id, count, topic_id, sorting_method, disable_check=True)

    def __eq__(self, other: object) -> bool:
        """判断是否是同一个小岛

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if not isinstance(other, Collection):
            return False  # 不是由小岛类构建的必定不相等
        if self._url == other._url:
            return True
        else:
            return False

    def __str__(self) -> str:
        """输出小岛信息摘要

        Returns:
            str: 小岛信息摘要
        """
        result = f"""小岛信息摘要：
小岛名：{self.name}
Url：{self.url}
分类：{self.category}
成员数：{self.members_count}
帖子数：{self.posts_count}
简介：\n{self.introduction}"""

        return result
